/* 
vim:expandtab:softtabstop=2:tabstop=2:shiftwidth=2:nowrap:ruler
*/
/*
Copyright (c) 2015-2016, iwrite authors 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

1. Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
#include "item_date.h"

extern void
item_date_assign(
  struct item_date*const                o_date,
  struct item_method*const              o_method)
{

  memset(o_date, 0, sizeof(*o_date));

  (*o_date).m_box.m_len_x= 30.0;
  (*o_date).m_box.m_len_y= 18.0;
  (*o_date).m_font= pango_font_description_new();
  (*o_date).m_color.alpha= 1.0;
  (*o_date).m_border.m_style= line_style_solid;
  (*o_date).m_border.m_thickness= 1.0;
  (*o_date).m_border.m_color.alpha= 1.0;
  (*o_date).m_fill.m_color.alpha= 1.0;
  (*o_date).m_date_style= date_style_short;

  ITEM_METHOD_ASSIGN(o_method, date);

  return;
}

extern void
item_date_copy(
  struct item_date *const               o_date,
  struct item_date const*const          i_date)
{

  memcpy(o_date, i_date, sizeof(*o_date));
  (*o_date).m_font= pango_font_description_copy((*i_date).m_font);
  (*o_date).m_locale= g_strdup((*i_date).m_locale); 
  (*o_date).m_timezone= g_strdup((*i_date).m_timezone); 

  return;
}

extern void
item_date_discharge(
  struct item_date*const                io_date)
{

  if ((*io_date).m_font)
  {
    pango_font_description_free((*io_date).m_font);
  }

  if ((*io_date).m_locale)
  {
    g_free((*io_date).m_locale);
  }

  if ((*io_date).m_timezone)
  {
    g_free((*io_date).m_timezone);
  }

  memset(io_date, 0, sizeof(*io_date));

  return;
}

extern void
item_date_draw(
  struct item_date*const                i_date,
  struct item_draw_context const*const  i_ctx)
{
  double                                l_center_x;
  double                                l_center_y;
  PangoFontDescription*                 l_font;
  PangoLayout*                          l_layout;
  cairo_matrix_t                        l_matrix;
  double                                l_radians;
  double                                l_shear_x;
  double                                l_shear_y;
  char*                                 l_text;

  l_font= 0;
  l_layout= 0;
  l_text= 0;

  do
  {

    if ((item_draw_design == (*i_ctx).m_option.m_type) &&
      (0 == (*i_date).m_time))
    {
      (*i_date).m_time= time(0);
    }

    icu_date_to_textual(
      &l_text,
      (*i_date).m_time,
      (*i_date).m_date_style,
      (*i_date).m_time_style,
      (*i_date).m_locale,
      (*i_date).m_timezone);

    cairo_new_path((*i_ctx).m_cr);
    cairo_save((*i_ctx).m_cr);
    cairo_scale((*i_ctx).m_cr, (*i_ctx).m_scale, (*i_ctx).m_scale);

    l_layout= pango_cairo_create_layout((*i_ctx).m_cr);
    pango_layout_set_width(l_layout, -1);
    pango_layout_set_text(l_layout, l_text, -1);
    pango_layout_set_font_description(l_layout, (*i_date).m_font); 

    l_center_x= (*i_date).m_box.m_pos_x + ((*i_date).m_box.m_len_x / 2.0);
    l_center_y= (*i_date).m_box.m_pos_y + ((*i_date).m_box.m_len_y / 2.0);

    cairo_translate((*i_ctx).m_cr, l_center_x, l_center_y);
    l_radians= geom_deg2rad((*i_date).m_transform.m_rotation);
    cairo_rotate((*i_ctx).m_cr, l_radians);
    cairo_translate((*i_ctx).m_cr, -l_center_x, -l_center_y);

    l_shear_x= tan(geom_deg2rad((*i_date).m_transform.m_shear_x));
    l_shear_y= tan(geom_deg2rad((*i_date).m_transform.m_shear_y));
    
    cairo_matrix_init(
      &l_matrix,
      1.0, l_shear_y,
      l_shear_x, 1.0,
      (*i_date).m_box.m_pos_x, (*i_date).m_box.m_pos_y);
    cairo_transform((*i_ctx).m_cr, &l_matrix);


    if ((*i_date).m_fill.m_active)
    {
      cairo_set_source_rgba(
        (*i_ctx).m_cr,
        (*i_date).m_fill.m_color.red,
        (*i_date).m_fill.m_color.green,
        (*i_date).m_fill.m_color.blue,
        (*i_date).m_fill.m_color.alpha);

      cairo_rectangle(
        (*i_ctx).m_cr,
        0.0,
        0.0,
        (*i_date).m_box.m_len_x,
        (*i_date).m_box.m_len_y);

      cairo_fill_preserve((*i_ctx).m_cr);
    }

    if ((*i_date).m_border.m_active)
    {

      item_common_cairo_dash_set((*i_ctx).m_cr, (*i_date).m_border.m_style);
      cairo_set_line_width((*i_ctx).m_cr, (*i_date).m_border.m_thickness);
      cairo_set_source_rgba(
        (*i_ctx).m_cr,
        (*i_date).m_border.m_color.red,
        (*i_date).m_border.m_color.green,
        (*i_date).m_border.m_color.blue,
        (*i_date).m_border.m_color.alpha);

      cairo_rectangle(
        (*i_ctx).m_cr,
        0.0,
        0.0,
        (*i_date).m_box.m_len_x,
        (*i_date).m_box.m_len_y);

      cairo_stroke_preserve((*i_ctx).m_cr);
    }

    cairo_set_source_rgba(
      (*i_ctx).m_cr,
      (*i_date).m_color.red,
      (*i_date).m_color.green,
      (*i_date).m_color.blue,
      (*i_date).m_color.alpha);

    pango_cairo_update_layout((*i_ctx).m_cr, l_layout);
    pango_cairo_show_layout((*i_ctx).m_cr, l_layout);

    if ((*i_ctx).m_option.m_highlight && 0 == (*i_ctx).m_selected)
    {
      item_common_cairo_dash_set((*i_ctx).m_cr, line_style_solid);
      cairo_set_line_width((*i_ctx).m_cr, 1.0);
      cairo_set_source_rgb((*i_ctx).m_cr, 0.50, 0.50, 0.00);
      cairo_rectangle(
        (*i_ctx).m_cr,
        0,
        0,
        (*i_date).m_box.m_len_x,
        (*i_date).m_box.m_len_y);
      cairo_stroke((*i_ctx).m_cr);
    }

    cairo_restore((*i_ctx).m_cr);

  }while(0);

  if (l_font)
  {
    pango_font_description_free(l_font);
  }

  if (l_layout)
  {
    g_object_unref(l_layout);
  }

  g_free(l_text);

  return;
}

extern void
item_date_get_bbox(
  struct geom_rect*const                o_bbox,
  cairo_t*                              io_cr,
  double const                          i_scale,
  struct item_date const*const          i_date)
{
  item_common_box_get_bbox(
    o_bbox, io_cr, &(*i_date).m_box, &(*i_date).m_transform, i_scale);
  return;
}

extern void
item_date_get_position(
  double*const                          o_pos_x,
  double*const                          o_pos_y,
  struct item_date const*const          i_date)
{

  (*o_pos_x)= (*i_date).m_box.m_pos_x;
  (*o_pos_y)= (*i_date).m_box.m_pos_y;

  return;
}

extern void
item_date_get_rotation(
  double*const                          o_rotation,
  struct item_date const*const          i_date)
{

  (*o_rotation)= (*i_date).m_transform.m_rotation;

  return;
}

extern void
item_date_get_size(
  double*const                          o_len_x,
  double*const                          o_len_y,
  struct item_date const*const          i_date)
{

  (*o_len_x)= (*i_date).m_box.m_len_x;
  (*o_len_y)= (*i_date).m_box.m_len_y;

  return;
}

extern void
item_date_get_shear(
  double*const                          o_shear_x,
  double*const                          o_shear_y,
  struct item_date const*const          i_date)
{

  (*o_shear_x)= (*i_date).m_transform.m_shear_x;
  (*o_shear_y)= (*i_date).m_transform.m_shear_y;

  return;
}

extern int
item_date_in_event(
  struct item_date const*const          i_date,
  cairo_t*                              io_cr,
  struct item_in_event const*const      i_ctx)
{
  int                                   l_hit;

  l_hit= item_common_box_in_event(
    io_cr, &(*i_date).m_box, &(*i_date).m_transform, i_ctx);

  return l_hit;
}

extern void
item_date_move(
  struct item_date*const                io_date,
  struct geom_point const*const         i_event)
{
  item_common_box_move(&(*io_date).m_box, i_event);
  return;
}

extern void
item_date_normalize(
  struct item_date*const                io_date)
{
  item_common_box_normalize(&(*io_date).m_box);
  return;
}

extern void
item_date_set_position(
  struct item_date*const                io_date,
  double const                          i_pos_x,
  double const                          i_pos_y)
{

  (*io_date).m_box.m_pos_x= i_pos_x;
  (*io_date).m_box.m_pos_y= i_pos_y;

  return;
}

extern void
item_date_set_rotation(
  struct item_date*const                io_date,
  double                                i_rotation)
{

  (*io_date).m_transform.m_rotation= i_rotation;

  return;
}

extern void
item_date_set_size(
  struct item_date*const                io_date,
  double                                i_len_x,
  double                                i_len_y)
{

  (*io_date).m_box.m_len_x= i_len_x;
  (*io_date).m_box.m_len_y= i_len_y;

  return;
}

extern void
item_date_set_shear(
  struct item_date*const                io_date,
  double                                i_shear_x,
  double                                i_shear_y)
{

  (*io_date).m_transform.m_shear_x= i_shear_x;
  (*io_date).m_transform.m_shear_y= i_shear_y;

  return;
}

extern int
item_date_read(
  GError**                              o_error,
  struct item_date*const                io_date,
  struct xml *const                     i_xml)
{
  GError*                               l_error;
  int                                   l_exit;
  int                                   l_found;
  GList*                                l_node;
  int                                   l_rc;
  GdkRGBA                               l_rgba;
  enum element_tag_type                 l_type;
  char const*                           l_value;
  struct xml*                           l_xml;

  l_error= 0;
  l_exit= 0;

  l_node= (*i_xml).m_children;

  do
  {

    if (0 == l_node)
    {
      break;
    }

    l_xml= (*l_node).data;

    item_common_lookup(&l_found, &l_type, (*l_xml).m_tag);
    g_strstrip((*l_xml).m_string->str);
    l_value= (*l_xml).m_string->str;

    if (l_found)
    {
      switch(l_type)
      {
        case element_tag_rect:
          item_common_box_read(&l_error, &(*io_date).m_box, l_xml);
          break;
        case element_tag_transform:
          item_common_read_transform(&l_error, &(*io_date).m_transform, l_xml);
          break;
        case element_tag_font:
          (*io_date).m_font= pango_font_description_from_string(l_value);
          break;
        case element_tag_color:
          l_rc= gdk_rgba_parse(&l_rgba, l_value);
          if (l_rc)
          {
            (*io_date).m_color= l_rgba;
          }
          break;
        case element_tag_border:
          l_exit= item_common_read_border(&l_error, &(*io_date).m_border, l_xml);
          break;
        case element_tag_fill:
          l_exit= item_common_read_fill(&l_error, &(*io_date).m_fill, l_xml);
          break;
        case element_tag_date_style:
          (*io_date).m_date_style= item_common_get_datetime_style(l_value);
          break;
        case element_tag_time_style:
          (*io_date).m_time_style= item_common_get_datetime_style(l_value);
          break;
        case element_tag_locale:
          (*io_date).m_locale= g_strdup(l_value);
          break;
        case element_tag_timezone:
          (*io_date).m_timezone= g_strdup(l_value);
          break;
        default:
          break;
      }
    }

    l_node= (*l_node).next;

  }while(1);

  if (l_error)
  {
    g_propagate_error(o_error, l_error);
  }

  return l_exit;
}

extern void
item_date_resize(
  struct item_date*const                io_date,
  struct item_resize_event const*const  i_ctx)
{

  item_common_box_resize(&(*io_date).m_box, i_ctx, &(*io_date).m_transform);

  return;
}

extern void
item_date_rotate(
  struct item_date*const                io_date,
  double const                          i_rotation)
{
  (*io_date).m_transform.m_rotation= i_rotation;
  return;
}
  
extern void
item_date_shear(
  struct item_date*const                io_date,
  double const                          i_delta_x,
  double const                          i_delta_y)
{

  (*io_date).m_transform.m_shear_x+= i_delta_x;
  (*io_date).m_transform.m_shear_y+= i_delta_y;

  return;
}

extern int
item_date_write(
  GError**                              o_error,
  FILE*const                            io_fp,
  struct item_date const*const          i_date)
{
  int                                   l_exit;
  char*                                 l_text;

  l_exit= 0;
  l_text= 0;

  fprintf(io_fp, "\t\t\t<item name=\"date\">\n");
  item_common_box_write(o_error, io_fp, &(*i_date).m_box);
  item_common_write_transform(o_error, io_fp, &(*i_date).m_transform);
  l_text= pango_font_description_to_string((*i_date).m_font);
  fprintf(io_fp, "\t\t\t\t<font>%s</font>\n", l_text);
  g_free(l_text);
  l_text= gdk_rgba_to_string(&(*i_date).m_color);
  fprintf(io_fp, "\t\t\t\t<color>%s</color>\n", l_text);
  g_free(l_text);
  item_common_write_border(o_error, io_fp, &(*i_date).m_border);
  item_common_write_fill(o_error, io_fp, &(*i_date).m_fill);
  fprintf(io_fp, "\t\t\t\t<date_style>%s</date_style>\n", item_common_get_datetime_style_text((*i_date).m_date_style));
  fprintf(io_fp, "\t\t\t\t<time_style>%s</time_style>\n", item_common_get_datetime_style_text((*i_date).m_time_style));

  if ((*i_date).m_locale && (*i_date).m_locale[0])
  {
    fprintf(io_fp, "\t\t\t\t<locale>%s</locale>\n", (*i_date).m_locale);
  }

  if ((*i_date).m_timezone && (*i_date).m_timezone[0])
  {
    fprintf(io_fp, "\t\t\t\t<timezone>%s</timezone>\n", (*i_date).m_timezone);
  }

  fprintf(io_fp, "\t\t\t</item>\n");

  return l_exit;
}
